#!/usr/bin/perl -w

# DAFARE:
# - evitare la visualizzazione, o farla funzionare bene; il formato grafico
#   pu essere dedotto dall'estensione del filename;
# - i parametri come opzioni, esclusi target e ora. I valori di default 
#   possono essere in un file di configurazione, o in variabili di ambiente.
# - Fare in modo che possano essere utilizzati dati da pi sessioni.  
use strict;
use Getopt::Std;
use Time::Local;
use Mysql;
use SQLjmeter;

sub usage {
    print "Usage:\tplot_delay <options> -[DJ] <target> <timestring>\n";
    print "Timestring: e.g \"2003-01-31 16:22:16\"";
    print "Options:\n\t-h <SQLserver>: SQL server address\n";
    print "\t-u <SQLuser>: SQL user name\n";
    print "\t-d <SQLdatabase>: SQL database\n";
    print "\t-p <probe>: probe address\n";
    print "\t-f <filename>: output file name (extension indicates format)\n";
    print "\t-l <lapse>: time interval before/after indicated time\n";
    print "\t-L: y scale is logarithmic\n";
    print "\t-P: display with points (default is line)\n";
    print "\t-D: plot backward/forward delay offset\n";
    print "\t-J: plot forward/backward jitter\n";
    print "\t-B: plot backward only statistic\n";
    print "\t-F: plot forward only statistic\n";
    print "\t-R: plot 2-way statistics\n";
    print "Available output format: ps, png, gif\n";
}

my %options;
my $i;

getopts('h:u:d:p:f:l:LDRJBFP',\%options);

my $SQLserver=( exists $options{'h'} )?$options{'h'}:"localhost";
my $SQLuser=( exists $options{'u'} )?$options{'u'}:"consumer";
my $SQLdatabase=( exists $options{'d'} )?$options{'d'}:"netmondb";
my $probe=( exists $options{'p'} )?$options{'p'}:"localhost";
my $file=$options{'f'};
my $lapse=( exists $options{'l'} )?$options{'l'}:"600";
my ( $format, $base);
my ( $target,$time) = @ARGV;

( defined $target ) || ( usage && die "\n*** Target host must be provided\n");
( defined $time ) || ( usage && die"\n*** Timestring must be provided\n");
( ( exists $options{'D'} ) || ( exists $options{'J'} ) ) ||
  ( usage && die "\n*** At least one of -D or -J must be given\n" );

# Both options are incompatible...
( ( exists $options{'B'} ) && ( exists $options{'F'} ) ) &&
  ( delete $options{'B'} && delete $options{'F'} );


if (defined $file) {
    unlink $file;
    my @parts=split /\./,$file;
    $format=$parts[$#parts];
    pop @parts;
    $base=join "\.",@parts;
#    print "$format $base\n";
}

my $dbh=SQLjmeter::connect($SQLserver,$SQLuser,$SQLdatabase) || 
    die "Cannot access database\n";
my ($tableh,$fa,$fg,$ba,$bg)=
    $dbh->SQLjmeter::getdelayoffset ($probe,$target,$time,$lapse);
my @table=@$tableh;

open PLOT, "> plot.gnu";

if (defined $file) {
  SWITCH: {
      if ( $format eq 'ps' ) { 
	  print PLOT "set output \"$file\"\n";
	  print PLOT "set terminal postscript landscape color\n";
	  last SWITCH;
      }
      if ( $format eq 'png' ) {
	  print PLOT "set output \"$file\"\n";
	  print PLOT "set terminal png\n";
	  last SWITCH;
      }
      if ( $format eq 'gif' ) {
	  print PLOT "set output \"$base.png\"\n";
	  print PLOT "set terminal png\n";
	  last SWITCH;
      }
  }
} else {
    print PLOT "set terminal X11\n";
}

if ( exists $options{'L'} ) {
    print PLOT "set logscale y\n";
}

print PLOT "set xdata time\n";
print PLOT "set timefmt \"%Y-%m-%d %H:%M:%S\"\n";
print PLOT "set format x '%d/%m/%y %H:%M:%S'\n";
print PLOT "set xtic rotate autofreq\n";
print PLOT "set mxtics 10\n";

print PLOT "plot ";
my $first=1;

sub plotline {
    if ( $first ) { $first=0; } else { print PLOT ", ";}  
    print PLOT " '-' using 1:3 title \"$_[0]\""; 
    if ( exists $options{'P'} ) { 
	print PLOT " with points"; 
    } else { 
	print PLOT " with line"; 
    }
}   
sub delayseries {
    for ($i=0;$i<=$#table;$i++) {
	my ($sec,$min,$hour,$mday,$mon,$year)=localtime($table[$i][0]);
	$mon++;
	$year+=1900;
	printf PLOT "%d-%d-%d %d:%d:%d\t%f\n",
	$year,$mon,$mday,$hour,$min,$sec,$table[$i][$_[0]];
    }
    print PLOT "e\n";
}
sub jitterseries {
    for ($i=0;$i<=$#table;$i++) {
	my ($sec,$min,$hour,$mday,$mon,$year)=localtime($table[$i][0]);
	$mon++;
	$year+=1900;
	printf PLOT "%d-%d-%d %d:%d:%d\t%f\n",
	$year,$mon,$mday,$hour,$min,$sec,$table[$i][$_[0]]-$table[$i-1][$_[0]];
    }
    print PLOT "e\n";
}

if ( exists $options{'D'} ) {
    if ( exists $options{'R'} ) {
	plotline("roundrip real delay");
    }
    if ( ! exists $options{'B'} ) {
	my $t=sprintf("forward offset delay (a=%2.5e,g=%5.0f)", $fa,$fg);
	plotline($t);
    }
    if ( ! exists $options{'F'} ) { 
	my $t=sprintf("backward offset delay (a=%2.5e,g=%5.0f)", $ba,$bg);
	plotline($t);
    }
}

if ( exists $options{'J'} ) {
    if ( exists $options{'R'} ) {
	plotline("roundrip jitter");
    }
    if ( ! exists $options{'B'} ) { 
	plotline("forward jitter");
    }
    if ( ! exists $options{'F'} ) { 
	plotline("backward jitter");
    }
}

print PLOT "\n";

if ( exists $options{'D'} ) {
    if ( exists $options{'R'} ) {
	delayseries(3);
    }
    if ( ! exists $options{'B'} ) { 
	delayseries(1);
    }
    if ( ! exists $options{'F'} ) { 
	delayseries(2);
    }
}

if ( exists $options{'J'} ) {
    if ( exists $options{'R'} ) {
	jitterseries(3);
    }
    if ( ! exists $options{'B'} ) { 
	jitterseries(1);
    }
    if ( ! exists $options{'F'} ) { 
	jitterseries(2);
    }
}

print PLOT "quit\n";

if (defined $file) {
    `gnuplot < plot.gnu`;
    if ( $format eq 'gif' ) {
	`convert $base.png $file 2> /dev/null`;
    }
} else {
    `gnuplot -persist < plot.gnu`;
}










